<?php
/**
* WoWRoster.net WoWRoster
*
* RaidTracker Library
*
* LICENSE: Licensed under the Creative Commons
*          "Attribution-NonCommercial-ShareAlike 2.5" license
*
* @copyright  2002-2007 WoWRoster.net
* @license    http://creativecommons.org/licenses/by-nc-sa/2.5   Creative Commons "Attribution-NonCommercial-ShareAlike 2.5"
* @link       http://www.wowroster.net
* @package    RaidTracker
*/

if( !defined('IN_ROSTER') )
{
    exit('Detected invalid access to this file!');
}

require_once ($addon['dir'] . 'inc/raidtrackerbase.class.php');
require_once (ROSTER_BASE . 'lib/functions.lib.php');
require_once (ROSTER_BASE . 'lib/item.php');
require_once (ROSTER_BASE . 'lib/update.lib.php');

class RaidTracker extends RaidTrackerBase {

    /**
    * GetRaidIcon(zone) - Returns the path and filename of the zone icon
    *
    * @param string zone
		* @param bool html - return html output or just image string 
    **/
    function GetRaidIcon($zone,$html = false) {
        global $roster, $addon;
        
        if(!isset($zone)) return false;
        
        // Pull the zones array out of the locale
        $zones = $roster->locale->act['raidtracker_zones'];

        // Loop through the zones and look for a match
        foreach( array_keys($zones) as $scan ) {
					if($scan == $zone && 
						file_exists( $addon['dir'] . $addon['config']['raidtracker_imageloc'] . DIR_SEP . $zones[$scan]['icon']. "." . $addon['config']['raidtracker_imgformat']) ) {
							if($html)
								return '<img src="'.$addon['url_full'] . $addon['config']['raidtracker_imageloc'] . DIR_SEP . $zones[$scan]['icon'] . "." . $addon['config']['raidtracker_imgformat'].'">';
							else
								return $addon['url_full'] . $addon['config']['raidtracker_imageloc'] . DIR_SEP . $zones[$scan]['icon'] . "." . $addon['config']['raidtracker_imgformat'];
						}
					else continue;
        }
				if($html)
			        return '<img src="'.$addon['url_full'] . $addon['config']['raidtracker_imageloc'] . DIR_SEP . "unknown." . $addon['config']['raidtracker_imgformat'].'">';
				else
			        return $addon['url_full'] . $addon['config']['raidtracker_imageloc'] . DIR_SEP . "unknown." . $addon['config']['raidtracker_imgformat'];
    }

    /**
    * GetBossIcon(boss) - Returns the path and filename of the boss icon
    *
    * @param string boss
		* @param bool html - return html output or just image string 
    **/
    function GetBossIcon($zone,$html = false) {
        global $roster, $addon;

        if(!isset($zone)) return false;
        
        // Pull the zones array out of the locale
        $zones = $roster->locale->act['raidtracker_zones'];
				$bossq = 'SELECT * FROM `' . $roster->db->table('bosslist','raidtracker') . '` WHERE `boss_locale` = "'
                           . $roster->config["locale"] . '";';
        $result = $roster->db->query($bossq);
        $bosses = $roster->db->fetch_all($result,SQL_ASSOC);

        // Loop through the zones and look for a match
        foreach( array_keys($bosses) as $scan ) {
					if($scan == $zone && 
						file_exists( $addon['dir'] . $addon['config']['raidtracker_imageloc'] . DIR_SEP . $zones[$scan]['icon']. "." . $addon['config']['raidtracker_imgformat']) ) {
							if($html)
								return '<img src="'.$addon['url_full'] . $addon['config']['raidtracker_imageloc'] . DIR_SEP . $zones[$scan]['icon'] . "." . $addon['config']['raidtracker_imgformat'].'">';
							else
								return $addon['url_full'] . $addon['config']['raidtracker_imageloc'] . DIR_SEP . $zones[$scan]['icon'] . "." . $addon['config']['raidtracker_imgformat'];
						}
					else continue;
        }
				if($html)
			        return '<img src="'.$addon['url_full'] . $addon['config']['raidtracker_imageloc'] . DIR_SEP . "unknown." . $addon['config']['raidtracker_imgformat'].'">';
				else
			        return $addon['url_full'] . $addon['config']['raidtracker_imageloc'] . DIR_SEP . "unknown." . $addon['config']['raidtracker_imgformat'];
    }

	/**
	* Returns the SQL where string for colors based on minquality
	*
	* @param int quality
	**/
	function GetColorSQL($quality = 0) {
		global $roster;

		if($quality > 0) {
			$where = " AND (";
			for($r = $quality; $r < sizeof(array_keys($roster->locale->act['lootTypes'])); $r++) {
				if($r == $quality)
					$where .= "l.color = '".$roster->locale->act['lootTypes'][$r]['color']."'";
				else
					$where .= " OR l.color = '".$roster->locale->act['lootTypes'][$r]['color']."'";
			}
			$where .= ")";
			return $where;
		} else return false;
	}
	
        /**
        * Creates a loot quality pulldown menu and a submit button
        *
        * @param int default - quality default
        **/
        function CreateLootPulldownForm($default = 0) {
                global $roster;
               
                $output = 	'<form name="qSelect" method="post" action="' . makelink() . '">
                <table>
                <tr><th colspan="2">'.$roster->locale->act['h_select_options'].'</th></tr>
                <tr><th class="membersHeader">'.$roster->locale->act['min_loot_quality'].'</th>
                <td><select name="quality">';
                for($r = 0; $r < sizeof(array_keys($roster->locale->act['lootTypes'])); $r++) {
                        if($r == $default)
                                $output .= '<option value="'.$r.'" selected="selected">'.$roster->locale->act['lootTypes'][$r]['desc'].'</option>';
                        else
                                $output .= '<option value="'.$r.'">'.$roster->locale->act['lootTypes'][$r]['desc'].'</option>';
                }
                $output .= '</select></td></tr></table></form>';
                return $output;
        }
        
        /**
        * Creates a loot quality pulldown menu
        *
        * @param int default - quality default
        * @param string form - name of the form
        **/
        function CreateLootPulldown($default = 0, $form = 'qSelect') {
                global $roster;
               
                $output = 	'
                <tr><th colspan="2">'.$roster->locale->act['h_select_options'].'<br /><font size="-3">'.$roster->locale->act['h_select_dates'].'</font></th></tr>
                <tr><th class="membersHeader">'.$roster->locale->act['min_loot_quality'].'</th>
                <td><select name="quality">';
                for($r = 0; $r < sizeof(array_keys($roster->locale->act['lootTypes'])); $r++) {
                        if($r == $default)
                                $output .= '<option value="'.$r.'" selected="selected">'.$roster->locale->act['lootTypes'][$r]['desc'].'</option>';
                        else
                                $output .= '<option value="'.$r.'">'.$roster->locale->act['lootTypes'][$r]['desc'].'</option>';
                }
                $output .= '</select></td></tr>';
                return $output;
        }

		/**
		* Creates a zone pulldown menu
		*
		* @param int default - zone default
		* @param string form - name of the form
		**/
		function CreateZonePulldown($default = 0, $form = 'qSelect') {
			global $roster;
			
			$output = '
				<tr><th colspan="2">'.$roster->locale->act['h_select_options'].'<br /><font size="-3">'.$roster->locale->act['h_select_dates'].'</font></th></tr>
				<tr><th class="membersHeader">'.$roster->locale->act['zone'].'</th>
				<td><select name="zone">';
			$res = $roster->db->query("SELECT `z`.`zone_id`,`z`.`zone_localized`,`z`.`zone_icon` FROM ".$roster->db->table('zonelist','raidtracker')." z ".
					"WHERE `z`.`zone_locale` = '".$roster->config['locale']."' ORDER BY `z`.`zone_localized` ASC");
			$zones = $roster->db->fetch_all($res, SQL_ASSOC);
			$output .= '<option value="0">--- '.$roster->locale->act['h_select_zone'].' ---</option>';
			for($r = 0; $r < sizeof($zones); $r++) {
				if($zones[$r]['zone_id'] == $default)
					$output .= '<option value="'.$zones[$r]['zone_id'].'" selected="selected">'.$zones[$r]['zone_localized'].'</option>';
				else
					$output .= '<option value="'.$zones[$r]['zone_id'].'">'.$zones[$r]['zone_localized'].'</option>';
			}
			$output .= '</select></td></tr>';
			return $output;
		}
		
        /**
        * Creates a date field menu
        *
        * @param string boxname - name of the form box
        * @param int default - default date string
        * @param string formname - the name of the overall form
        * @param string mystr - the locale string of the date select
        **/
        function CreateDateBox($boxname, $default = '0000-00-00', $formname = 'qDate', $mystr = 'start') {
        		global $roster;
        		
        		$output = '
        		<tr><th class="membersHeader"><a href="#" onclick="cal.select(document.forms[\''.$formname.'\'].'.$boxname.', \''.$mystr.'\', \'yyyy-MM-dd\'); return false;" NAME="'.$mystr.'" ID="'.$mystr.'">'.$roster->locale->act[$mystr.'_date'].'</a></th>
        		<td><input id="'.$boxname.'" name="'.$boxname.'" value="'.$default.'" />
        		</td></tr>';
        		return $output;
        }

		/**
		* Create a loot and date selection box
		*
		* @param string formname
		* @param int default quality
		* @param string start date
		* @param string end date
		**/
		function CreateLootSelectBox($formname = 'newform',$defq, $start = '0000-00-00', $end = '0000-00-00') {
			$output = '<form name="qSelect" method="post" action="' . makelink() . '"><table>'.
			$this->CreateLootPulldown($defq, $formname).
			$this->CreateDateBox('startdate', $start, $formname,'start').
			$this->CreateDateBox('enddate',$end,$formname, 'end').
			'<tr><td colspan="2"><input type="Submit" /></td>'.
			'</table></form>';
			return $output;
		}

		/**
		* Create a zone and date selection box
		*
		* @param string formname
		* @param int default zone
		* @param string start date
		* @param string end date
		**/
		function CreateZoneSelectBox($formname = 'newform', $defz, $start = '0000-00-00', $end = '0000-00-00') {
			$output = '<form name="qSelect" method="post" action="'.makelink().'"><table>'.
			$this->CreateZonePulldown($defz,$formname).
			$this->CreateDateBox('startdate', $start, $formname, 'start').
			$this->CreateDateBox('enddate', $end, $formname, 'end').
			'<tr><td colspan="2"><input type="Submit" /></td>'.
			'</table></form>';
			return $output;
		}
}

class RaidTrackerItem extends item
{
        /**
         * Takes the current item data and caches it to the database
         * @param array $data
         */
        function _cacheArmoryItem( $item_id = FALSE, $fullitem = FALSE, $tip = FALSE )
        {
                global $roster;
               
								// If we received data, then we need to set the item struct
								// and prepare it for caching.
								if( $item_id !== FALSE && $fullitem !== FALSE && $tip !== FALSE ) {
										// Set the item variables so we can parse the tooltip
										$this->icon = $fullitem->itemInfo->item->icon;
										$this->item_id = $item_id;
										$this->level = $fullitem->itemInfo->item->level;
										$this->name = $fullitem->itemInfo->item->name;
										$this->color = $this->_getColorByQuality( $fullitem->itemInfo->item->quality );
										$this->tooltip = $tip;
										$this->_setQuality( $this->color );
										$this->locale = $roster->config['locale'];
										$this->_doParseTooltip();
										$this->_makeTooltipHTML();
								}
								
                // Make sure that the item doesn't already exist. If it does, then we
                // need to update, not insert.
                $query = 'SELECT * FROM `' . $roster->db->table('itemcache','raidtracker') . '` WHERE `wow_id` = "'
                           . $this->item_id . '" LIMIT 1;';
                $check = $roster->db->query_first($query);
 
                if( empty($check) )
                {
                        $build = $roster->db->build_query('INSERT',array(
                                'item_id'      => 'NULL',
                                'wow_id'       => $this->item_id,
                                'item_name'    => $roster->db->escape($this->name),
                                'item_color'   => $this->color,
                                'item_texture' => strtolower($this->icon),
                                'item_tooltip' => $this->tooltip,
                                'html_tooltip' => $this->html_tooltip,
                                'item_level'   => $this->level,
                                'locale'       => $this->locale,
//                        'last_update' => 'TIMESTAMP(NOW())'
                                )
                        );
               
                        $querystr = "INSERT INTO `" . $roster->db->table('itemcache','raidtracker') . "` " . $build . ';';
                }
                else
                {
                        $build = $roster->db->build_query('UPDATE',array(
                                'item_name'    => $roster->db->escape($this->name),
                                'item_color'   => $this->color,
                                'item_texture' => strtolower($this->icon),
                                'item_tooltip' => $this->tooltip,
                                'html_tooltip' => $this->html_tooltip,
                                'item_level'   => $this->level,
                                'locale'       => $this->locale,
//                        'last_update' => 'TIMESTAMP(NOW())'      
                                )
                        );
 
                        $querystr = "UPDATE `" . $roster->db->table('itemcache','raidtracker') . "` SET " . $build . ' WHERE `wow_id` = "' . $this->item_id . '";';
                }
                //echo "<br>cache call: $querystr</br>";
 
                $result = $roster->db->query($querystr);
                if( !$result )
                {
                        echo '<ul>' . $roster->locale->act['rt_item_insert_fail'] . '</ul>';
                        echo $roster->db->error();
                }
                //echo "<br>Inserted $this->item_name ($this->wow_id) into the cache.<br>";
                return;
        }
 
        /**
         * Function _getQualityByColor($color)
         *
         * Private function for returning item quality by given
         * color string.
         */
        function _getQualityByColor( $color )
        {
                global $roster;
                $types = $roster->locale->act['lootTypes'];
                foreach( array_keys($types) as $ckey )
                {
                        // Extract the color we're currently working with
                        if( $types[$ckey]['color'] == $color )
                        {
                                return $ckey;
                        }
        }
                // If we reach this point there was no color match
                return false;
        }
 
        /**
         * Function _getColorByQuality($quality)
         *
         * Private function for returning item color by given
         * quantity number.
         */
        function _getColorByQuality( $quality )
        {
                global $roster;
                return $roster->locale->act['lootTypes'][$quality]['color'];
        }
 
        /**
         * Function _itemExists( $item_id )
         *
         * @param $item_id - string
         *
         * Private function for validating an item in the raidtracker
         * item cache.
         *
         * Returns false if the item either does not exist, or is past
         * the expiration timer.
         */
        function _itemExists( $item_id )
        {
                global $roster,$addon;
               
                $expire_limit = $addon['config']['raidtracker_itemcache_expires'];
               
                $query = 'SELECT * FROM `' . $roster->db->table('itemcache','raidtracker') . '` WHERE `wow_id` = "'
                           . $item_id . '" AND ABS( DATEDIFF( NOW(),`last_update`) ) < ' . $expire_limit . ' LIMIT 1;';
                $result = $roster->db->query_first($query);
               
                if( !empty($result) )
                {
                        return true;
                }
                return false;
        }
 
        /**
         * Loads a cached item, sets the item variables in the object
         * and returns the formatted icon/item tooltip.
         *
         * @param string item_id = WoW item ID
         */
        function _loadCachedItem( $item_id )
        {
                global $roster;
               
                // Retrieve the item. We already know it exists.
                $query = 'SELECT * FROM `' . $roster->db->table('itemcache','raidtracker') . '` WHERE `wow_id` = "' . $item_id . '" LIMIT 1;';
                $ret = $roster->db->query($query);
               
                if( $ret )
                {
                        $result = $roster->db->fetch_all($ret,SQL_ASSOC);
                }
                $roster->db->free_result($ret);
 
                // Set the item variables so we can parse the tooltip
                $result = $result[0];
                $this->icon = $result['item_texture'];
                $this->item_id = $result['wow_id'];
                $this->level = $result['item_level'];
                $this->name = $result['item_name'];
                $this->color = $result['item_color'];
                $this->tooltip = $result['item_tooltip'];
                $this->html_tooltip = $result['html_tooltip'] . '<br />&lt;&lt;Cached ' . $result['last_update'] . '&gt;&gt;';
                $this->_setQuality( $this->color );
                $this->locale = $roster->config['locale'];
 
//            $this->_doParseTooltip();
//            $this->_makeTooltipHTML();
								unset($result);
                return $this->out();
        }

				/**
				* Loads a cached item info and returns it in an array
				*
				* Call _itemExists() first!!!
				**/
				function _loadCachedItemInfo( $item_id ) {
						global $roster;
						
						$query = 'SELECT * FROM `' . $roster->db->table('itemcache','raidtracker') . '` WHERE `wow_id` = "' . $item_id . '" LIMIT 1;';
						$ret = $roster->db->query($query);
						
						if( $ret ) {
								$result = $roster->db->fetch_all($ret,SQL_ASSOC);
						}
						$roster->db->free_result($ret);
						
						$result = $result[0];
						
						return $result;
				}

				/**
				* Takes a WoW ID and returns the HTML item name
				**/
				function fetchItemNameHTML( $item_id ) {
						global $roster;
						
//						echo "fetchItemNameHTML: Received $item_id<br>";
						// First see if the item is in the cache
						if($this->_itemExists( $item_id ) ) {
								$data = $this->_loadCachedItemInfo( $item_id );
								
								$name = $data['item_name'];
								$color = $data['item_color'];
								return "<div><font color='#".substr($color,2)."'>".stripslashes($name)."</font><font size='-5'><br>(cached)</font></div>";
								
						} else {
								// Use the armory
								$this->_initArmoryDb();
								$this->armory_db->setTimeOut(8);
								
								$data = $this->armory_db->fetchItemInfo( array_shift( explode( ':', $item_id ) ), $roster->config['locale'], 'simpleClass' );
								$tip = $this->fetchArmoryTooltip($item_id);
								if(!empty($data) && !empty($tip)) {
										$color = $this->_getColorByQuality( $data->itemInfo->item->quality );
										$this->_cacheArmoryItem($item_id, $data, $tip);
										return "<div><font color='#".substr($color,2)."'>".$data->itemInfo->item->name."</font><br><font size='-5'>(armory)</font></div>";
								} else {
										return "Armory data not received";
								}
						}
				}
				
        /**
         * Takes a WoW ID and the item name and attempts to find
         * the item in the member's item cache, the raidtracker
         * item cache.
         *
         * If the item does not exist in either of those, or the
         * item has passed the cached expired limit, it attempts
         * to retrieve it from the armory.
         *
         * This function returns the icon and tooltip for the
         * requested item.
         *
         * @param string $item_id = WoW item ID
         * @param string $itemname = WoW item name
         *
         */
        function fetchItemOrArmory( $item_id , $itemname = "" )
        {
                global $roster;
 
                // First try to fetch it from the members' item database
                $data = $this->fetchItemID( $item_id );
                if( empty( $data ) )
                {
                        // echo "fetchItemOrArmory: Failed to fetch data from database for item_id ".$item_id."<br>";
                        // Now see if it exists in the item cache and is still recent.
                        // If it does, we display the tooltip
                        if( $this->_itemExists($item_id) )
                        {
                                return $this->_loadCachedItem($item_id);
                        }
                        else
                        {
                                // Otherwise we need to fetch the item from the armory
                                // and cache it.
                                $this->_initArmoryDb();
                                $this->armory_db->setTimeOut(8);
//                        $this->armory_db->region = 'EU';
       
                                $id = array_shift( explode( ':', $item_id ) );
                                $fullitem = $this->armory_db->fetchItemInfo($id, $roster->config['locale'], 'simpleClass');
                                $tip = $this->fetchArmoryTooltip($item_id);
 
                                if( !empty($tip) && !empty($fullitem) )
                                {
                                        //echo "<br>Tooltip raw: ";
                                        //aprint($tip);
       
                                        // Set the item variables so we can parse the tooltip
                                        $this->icon = $fullitem->itemInfo->item->icon;
                                        $this->item_id = $item_id;
                                        $this->level = $fullitem->itemInfo->item->level;
                                        $this->name = $fullitem->itemInfo->item->name;
                                        $this->color = $this->_getColorByQuality( $fullitem->itemInfo->item->quality );
                                        $this->tooltip = $tip;
                                        $this->_setQuality( $this->color );
                                        $this->locale = $roster->config['locale'];
                                        $this->_doParseTooltip();
                                        $this->_makeTooltipHTML();
 
                                        // echo "<br>Full item:<br>";
                                        // aprint($fullitem);
                                        //echo '<br>Done ' . $item_id . '<br>';
                                        unset($tip);
                                        $this->_cacheArmoryItem();
                                        //echo $this->out();
                                        return $this->out();
                                }
                                else
                                {
                                        // We failed to retrieve the armory data
                                        $this->icon = 'inv_misc_questionmark';
                                        $this->name = 'Failed to Retrieve from Armory';
                                        $this->level = 0;
                                        $this->tooltip = 'Failed to retrieve item data for ' . $id . ' from the armory. Please try again later.<br>';
                                        $this->color = $this->_getColorByQuality( 0 );
                                        $this->item_id = '0';
                                        $this->html_tooltip = '<span>Failed to retrieve item data for ' . $id . ' from the armory. Please try again later.</span><br>';
                                        return $this->out();
                                }
                        }
                }
                else
                {
                        //echo "fetchItemOrArmory: Dumping stored data for item_id ".$item_id.":<br>";
                        return $this->printTooltipIcon($itemname);
                        //echo "<br>";
                }
                return;
        }
 
        /**
         * This function graciously borrowed from armorysync
         */
        function fetchArmoryTooltip( $item_id )
        {
                global $roster;
               
                $this->_initArmoryDb();
                $this->armory_db->setTimeOut(8);
//            $this->armory_db->region = 'EU';
 
                $id = array_shift( explode( ":", $item_id ) );
 
                $content = $this->armory_db->fetchItemTooltipHTML($id, $roster->config['locale']);
 
                if( !empty($content) )
                {
                        $html = $content;
            $content = str_replace("\n", '', $content );
            $content = str_replace("\r", '', $content );
                        $content = str_replace("\t", '', $content );
//            $content = str_replace('<span class="tooltipRight">', "\t", $content );
                        $content = preg_replace('/<img class="socketImg p".*?>(.*?)<br>/', '|cffffffff${1}|r<br>', $content );
//                  $content = preg_replace('/(?<=<span class="">)(\+\d+&nbsp;</span><span class="">\w+)(?=</span>)/' , '|cffffffff${1}|r', $content );
                        $content = str_replace("<br/>", '%__BRTAG%', $content );
            $content = str_replace("<br>", '%__BRTAG%', $content );
                        $content = str_replace('&nbsp;', ' ', $content );
                        $content = preg_replace('/\s\s+/', '', $content );
 
            // This is an ugly workaround for an encoding error in the armory
                        $content = preg_replace('/J.{3}ger/', 'Jäger', $content );
            // This is an ugly workaround for an encoding error in the armory
 
                        $tmpA = explode( '%__BRTAG%', $content);
 
                        $content = '';
                        foreach( $tmpA as $tmp )
                        {
                                $tmp = preg_replace( '/(.*?)<span class="tooltipRight">(.*?)<\/span>(.*)/', "\${1}\${3}\t\${2}", $tmp );
                                $content .= $tmp . '%__BRTAG%';
                        }
 
            $content = strip_tags($content);
                        $content = str_replace('%__BRTAG%', "\n", $content );
                        $content = preg_replace('/\s+\n/', "\n", $content );
            $content = utf8_encode($this->_unhtmlentities( $content ));
//                  $content = str_replace($roster->locale->act['bindings']['bind_on_pickup'], $roster->locale->act['bindings']['bind'], $content);
//                  $content = str_replace($roster->locale->act['bindings']['bind_on_equip'], $roster->locale->act['bindings']['bind'], $content);
 
                        if( $roster->locale->curlocale == 'frFR' )
                        {
                                $content = preg_replace( '/(\d+) Armure/', "Armure: \${1}", $content );
                                $content = preg_replace( '/(\d+) Blocage/', "Bloquer: \${1}", $content );
                                $content = preg_replace( '/Classes : (.+)/', "Classes  : \${1}", $content );
                                $content = preg_replace( '/Durabilit.*?:/', utf8_encode('Durabilité:'), $content);
                                $content = preg_replace( '/\+\s(\d+)/', "+\${1}", $content );
 
                        }
 
                        return $content;
                }
                else
                {
                        return false;
                }
        }
 
        /**
         * This function graciously borrowed from armorysync
         */
    function _unhtmlentities($string)
    {
        // Ersetzen numerischer Darstellungen
        $string = preg_replace('~&#x([0-9a-f]+);~ei', 'chr(hexdec("\\1"))', $string);
        $string = preg_replace('~&#([0-9]+);~e', 'chr("\\1")', $string);
        // Ersetzen benannter Zeichen
        $trans_tbl = get_html_translation_table(HTML_ENTITIES);
        $trans_tbl = array_flip($trans_tbl);
        $ret = strtr($string, $trans_tbl);
//            $this->_debug( 3, $ret, 'Removed HTML entities', 'OK' );
        return $ret;
    }
} 
?>
