<?php
/**
* WoWRoster.net WoWRoster
*
* RaidTracker Library
*
* LICENSE: Licensed under the Creative Commons
*          "Attribution-NonCommercial-ShareAlike 2.5" license
*
* @copyright  2002-2007 WoWRoster.net
* @license    http://creativecommons.org/licenses/by-nc-sa/2.5   Creative Commons "Attribution-NonCommercial-ShareAlike 2.5"
* @link       http://www.wowroster.net
* @package    RaidTracker
*/

if( !defined('IN_ROSTER') )
{
    exit('Detected invalid access to this file!');
}

if( !active_addon('memberslist') )
{
        // Memberslist not installed. Just die for now.
        roster_die( "You will need to install memberslist to use the raidtracker addon display component" );
}

// Include the memberslist class
$memberslist_addon = getaddon('memberslist');

include_once($memberslist_addon['dir'] . 'inc/memberslist.php');
include_once(ROSTER_LIB . 'item.php');

$display = new memberslist(array(), $memberslist_addon);

class RaidTrackerBase {


    /**
    * function DrawMenu($open) - Draws the overhead menu for the raidtracker
    *
    * @param open - boolean for window open/closed
    **/
    
    /**
    * function GetRaids() 	- Retrive raid data from the database by
    *				current context
    *
    * @param type 		- string = realm, char, guild
    * @param limit		- integer, number of records to be displayed
    * @param start		- integer, record start for display
    * @param deleted		- boolean, show deleted raids no/yes
    *
    **/
    function GetRaids($type = "guild", $limit = NULL, $start = 0, $deleted = 0) {
	    global $roster;
	    
	    // Set the base query constructor
	    $query = 	"SELECT r.*, COUNT(b.boss) AS bosscount FROM `".$roster->db->table('raids','raidtracker')."` r".
	    		" LEFT JOIN `".$roster->db->table('bosskills','raidtracker')."` b ON b.raidFK = r.raidID";
	    // Construct the query
	    switch($type) {
		    case "realm": {
			$where = " WHERE r.realm = '".$roster->data['server']."'";
			$group = " GROUP BY r.raidtime";
			break;
		    }
		    case "char": {
			$join = " INNER JOIN `".$roster->db->table('raidjoins','raidtracker')."` j ON j.raidFK = r.raidID";
			$where = " WHERE j.member = '".$roster->data['name']."' AND r.realm = '".$roster->data['server']."' AND r.faction = '".$roster->data['faction']."'";
			$group = " GROUP BY r.raidtime";
			break;
		    }
		    case "guild":
		    default: {
			// Extract the necessary information
			$where = " WHERE r.guild_name = '".$roster->data['guild_name']."' AND r.realm = '".$roster->data['server']."' AND r.faction = '".$roster->data['faction']."'";
			$group = " GROUP BY r.raidtime";
			break;
		    }
	    }
	    // If join exists, add it
	    if(isset($join)) $query .= $join;
	    // Add the WHERE
	    $query .= $where;
	    // If we're flagged to show deleted, do so
	    if($deleted == 0) $query .= " AND r.deleted = '0'";
	    // Add the grouping
	    $query .= $group;
	    // Now set the order and the limits
	    $query .= " ORDER BY r.raidtime DESC";
	    if(isset($limit)) $query .= " LIMIT ".$start.",".$limit;

	    // Debug
	    echo "<br>DEBUG: ".$query."<br>";
	    // Now perform the query and return the results
/*
	    $result = $roster->db->query($query);
	    if(!$result) {
		roster_die("MySQL error retrieving raids! ".$roster->db->error()."<br>","Raid Retrieval Error!");
	    } else {
		$data = $roster->db->fetch_all($result);
		unset($group,$where,$query);
		$roster->db->free_result($result);
		return $data;
	    }
*/
    }
    /**
    * function GetBosses() 	- Retrive boss data from the database by
    *				current context
    *
    * @param type 		- string = realm, char, guild
    * @param limit		- integer, number of records to be displayed
    * @param start		- integer, record start for display
    * @param deleted		- boolean, show deleted raids no/yes
    *
    **/
    function GetBosses($type = "guild", $limit = NULL, $start = 0, $deleted = 0) {
	    global $roster;
	    
	    // Set the base query constructor
	    $query = 	"SELECT b.*,r.zone FROM `".$roster->db->table('bosskills','raidtracker')."` b".
	    		" LEFT JOIN `".$roster->db->table('raids','raidtracker')."` r ON r.raidID = b.raidFK";
	    // Construct the query
	    switch($type) {
		    case "realm": {
			$where = " WHERE r.realm = '".$roster->data['server']."'";
			$group = " GROUP BY r.raidtime";
			break;
		    }
		    case "char": {
			$join = " INNER JOIN `".$roster->db->table('raidjoins','raidtracker')."` j ON j.raidFK = b.raidFK";
			$where = " WHERE j.member = '".$roster->data['name']."' AND r.realm = '".$roster->data['server']."' AND r.faction = '".$roster->data['faction']."'";
			$group = " GROUP BY r.raidtime";
			break;
		    }
		    case "guild":
		    default: {
			// Extract the necessary information
			$where = " WHERE r.guild_name = '".$roster->data['guild_name']."' AND r.realm = '".$roster->data['server']."' AND r.faction = '".$roster->data['faction']."'";
			$group = " GROUP BY r.raidtime";
			break;
		    }
	    }
	    // If join exists, add it
	    if(isset($join)) $query .= $join;
	    // Add the WHERE
	    $query .= $where;
	    // If we're flagged to show deleted, do so
	    if($deleted == 0) $query .= " AND b.deleted = '0'";
	    // Add the grouping
	    $query .= $group;
	    // Now set the order and the limits
	    $query .= " ORDER BY r.zone ASC,b.boss ASC";
	    if(isset($limit)) $query .= " LIMIT ".$start.",".$limit;

	    // Debug
	    echo "<br>DEBUG: ".$query."<br>";
	    // Now perform the query and return the results
/*
	    $result = $roster->db->query($query);
	    if(!$result) {
		roster_die("MySQL error retrieving raids! ".$roster->db->error()."<br>","Raid Retrieval Error!");
	    } else {
		$data = $roster->db->fetch_all($result);
		unset($group,$where,$query);
		$roster->db->free_result($result);
		return $data;
	    }
*/
    }
    /**
    * function GetLoot() 	- Retrive loot data from the database by
    *				current context
    *
    * @param type 		- string = realm, char, guild
    * @param quality 		- string = NULL (no restriction), or a quality
    *				number.
    * @param raidid 		- int, raidID if for specific raid
    * @param limit		- integer, number of records to be displayed
    * @param start		- integer, record start for display
    * @param deleted		- boolean, show deleted raids no/yes
    *
    **/
    function GetLoot($type = "guild", $quality = NULL, $raidid = NULL, $limit = NULL, $start = 0, $deleted = 0) {
	    global $roster;
	    
	    // Set the base query constructor
	    $query = 	"SELECT l.*,r.raidID,p.guild_name FROM `".$roster->db->table('items','raidtracker')."` l".
	    		" LEFT JOIN `".$roster->db->table('raids','raidtracker')."` r ON r.raidID = l.raidFK".
			" JOIN `".$roster->db->table('raidmembers', 'raidtracker')."` p ON p.name = l.member";
	    // Construct the query
	    switch($type) {
		    case "realm": {
			$where = " WHERE r.realm = '".$roster->data['server']."'";
			$group = " GROUP BY l.itemname ASC";
			break;
		    }
		    case "char": {
			$where = " WHERE l.member = '".$roster->data['name']."' AND r.realm = '".$roster->data['server']."' AND r.faction = '".$roster->data['faction']."'";
			$group = " GROUP BY l.itemname ASC";
			break;
		    }
		    case "guild":
		    default: {
			// Extract the necessary information
			$where = " WHERE p.guild_name = '".$roster->data['guild_name']."' AND r.realm = '".$roster->data['server']."' AND r.faction = '".$roster->data['faction']."'";
			$group = " GROUP BY l.itemname ASC";
			break;
		    }
	    }
	    // If the raidid is set, add it
	    if(isset($raidid)) $where .= " AND r.raidID = '".$raidid."'";
	    // If the quality is set, add quality qualifiers for that quality and higher
	    if(isset($quality)) {
		    $where .= " AND (";
		    for($r = $quality; $r < sizeof(array_keys($roster->locale->act['lootTypes'])); $r++) {
			    if($r == $quality)
				$where .= "l.color = '".$roster->locale->act['lootTypes'][$r]['color']."'";
			    else
			    	$where .= " OR l.color = '".$roster->locale->act['lootTypes'][$r]['color']."'";
		    }
		    $where .= ")";
	    }
	    // If join exists, add it
	    if(isset($join)) $query .= $join;
	    // Add the WHERE
	    $query .= $where;
	    // If we're flagged to show deleted, do so
	    if($deleted == 0) $query .= " AND l.deleted = '0'";
	    // Add the grouping
	    $query .= $group;
	    // Now set the order and the limits
	    $query .= " ORDER BY r.raidtime ASC,l.itemname ASC";
	    if(isset($limit)) $query .= " LIMIT ".$start.",".$limit;

	    // Debug
	    echo "<br>DEBUG: ".$query."<br>";
	    // Now perform the query and return the results

	    $result = $roster->db->query($query);
	    if(!$result) {
		roster_die("MySQL error retrieving raids! ".$roster->db->error()."<br>","Raid Retrieval Error!");
	    } else {
		$data = $roster->db->fetch_all($result);
		unset($group,$where,$query,$join,$r);
		$roster->db->free_result($result);
		return $data;
	    }
    }
}
?>
