<?php
/**
* WoWRoster.net WoWRoster
*
* LICENSE: Licensed under the Creative Commons
*          "Attribution-NonCommercial-ShareAlike 2.5" license
*
* @copyright  2002-2007 WoWRoster.net
* @license    http://creativecommons.org/licenses/by-nc-sa/2.5   Creative Commons "Attribution-NonCommercial-ShareAlike 2.5"
* @link       http://www.wowroster.net
* @package    RaidTracker
* @subpackage UpdateHook
*/

if ( !defined('IN_ROSTER') )
{
    exit('Detected invalid access to this file!');
}

/**
* Addon Update class
* This MUST be the same name as the addon basename
*
* @package    RaidTracker
* @subpackage UpdateHook
*/
class raidtrackerUpdate
{
    var $messages = '';        // Update messages
    var $data = array();    // Addon config data automatically pulled from the addon_config table
    var $files = array();    // Additional LUA files, we expect ct_raidtracker data
    var $chars = array();    // Uploaded characters
    var $sqls = array();    // SQL strings for injection. Indexed. Must be this way until
                    // we are able to deal with transactions since an injected primary
                // raidnum must be cross-referenced in the additional tables of data
    var $guild = '';
    var $region = '';
    var $server = '';
    var $faction = '';
    var $guildid = 0;
    
    // Global counts
    var $tbosses = 0;        // Total number of bosses
    var $tloots = 0;        // Total number of loots
    var $twipes = 0;        // Total number of wipes

    /**
     * Class instantiation
     * The name of this function MUST be the same name as the class name
     *
     * @param array $data    | Addon data
     * @return raidtracker
     */
    function raidtrackerUpdate( $data )
    {
        $this->data = $data;
        $this->files[] = 'ct_raidtracker';
    }

    // For now guild and realm identification is done in two ways, by user
    // context, or by default guild set in options. An upcomming version of
    // the modified raidtracker should store this information in the lua file
    // which will make it easier to track.
    //
    // Authentication is done via roster's internal. It is configurable between
    // the accepted user levels.
    /**
     * General update hook
     *
     * @return bool
     */

     function update() {
        global $roster, $update;
    
        $roster_login = new RosterLogin();
        // Make sure the addon is enabled and the user authorized
        if( ($this->data['config']['raidtracker_enable'] != true) ) {
        // The raidtracker is not enabled
        $this->messages .= '<strong>'.$roster->locale->act['rt_global_disable'].'</strong>';
        return;
        }
        if(!$roster_login->getAuthorized($this->data['config']['raidtracker_uploadaccess']) ) {
        // They do not have access
        $this->messages .= '<strong>Access denied!</strong>:'.$roster_login->getMessage();
        return; // Back to the framework
        }

    $raiddata = $update->uploadData['ct_raidtracker']['CT_RaidTracker_RaidLog'];

    $this->reset_messages();

        // Extract the realm information
    if(empty($roster->data['guild_name'])) {
        $q = "SELECT guild_name, server, region, faction FROM `".$roster->db->table('guild'). "` WHERE `guild_id` = ".$this->data['config']['raidtracker_default_guild'].";";
        $result = $roster->db->query($q);
        if(!$result) {
            $this->messages .= "<strong>".$roster->locale->act['rt_no_guildinfo']."</strong><br>";
            $this->messages .= $roster->db->error()."<br>";
            return false;
        } else {
            $guild = $roster->db->fetch($result, SQL_ASSOC);
            $this->guild = $guild['guild_name'];
            $this->region = $guild['region'];
            $this->server = $guild['server'];
            $this->faction = $guild['faction'];
            $this->guildid = $this->data['config']['raidtracker_default_guild'];
            $roster->db->free_result($result);
            unset($guild);
            unset($q);
        }
    } else {
        $this->guild = $roster->data['guild_name'];
        $this->region = $roster->data['region'];
        $this->server = $roster->data['server'];
        $this->faction = $roster->data['faction'];
        $this->guildid = $roster->data['guild_id'];
    }

    $this->messages .= '<strong>'.sprintf($roster->locale->act['upload_data'], 'CT_RaidTracker_RaidLog',$this->guild,$this->server,$this->region) . '</strong><br>';

        // Debugging
        $this->raidtracker_debug('Updating '.$this->faction.' '.$this->guild.'@'.$this->server.'-'.$this->region.'(Guild #'.$this->guildid.')<br>',1);
        
        // Version validation hook for if/when CT_Raidtracker puts this info in
    /*
        if( version_compare($char['version'], $this->data['config']['raidtracker_minver'], '>=') ) {
            $this->messages .= '<strong>' . sprintf($roster->locale->act['upload_data'],'RaidTrackerData',$char_name,$realm_name,$region) . "</strong>\n";
            $this->messages .= "<ul>\n";
            $this->messages .= "</ul>\n";
        }
        else // RaidTracker version not high enough
        {
            $this->messages .= '<span class="red">' . sprintf($roster->locale->act['not_updating'],'RaidTrackerData',$char_name,$char['version']) . "</span><br />\n";
            $this->messages .= sprintf($roster->locale->act['RaidTracker_err'], $this->data['config']['raidtracker_minver']) . "\n";
        }
    */

        // Process the raids
        $this->processRaids($raiddata);
    $this->raidtracker_debug("Memory Usage: " . round((memory_get_usage()/1048576),2) . " MB<br>",0);
    $this->raidtracker_debug("Processed ".$this->tbosses." bosses, ".$this->tloots." loots, ".$this->twipes." wipes from the file<br>",0);
        return true;
    }

    /**
     * Separates the entire raid data array into chunks and processes them
     *
     * @param array $raiddata
     */
    function processRaids(&$raiddata) {
        global $roster, $update;

        // Put each raid in an array and process it
        $raid_array = array();
        foreach( array_keys($raiddata) as $raid ) {
        $data_raid[$raid] = $raiddata[$raid];
        }

        $this->messages .= str_replace("%n", sizeof($data_raid), $roster->locale->act['rt_found_num_raids'])."<br>";
        foreach( array_keys($data_raid) as $raidId ) {
        $raid_array[$raidId] = $data_raid[$raidId];
        $msg = $roster->locale->act['rt_processing_raid'];
        $msg = str_replace("%r", $raidId, $msg);
        $msg = str_replace("%n", sizeof($data_raid), $msg);
        $this->messages .= $msg.'<br>';
        // Process this raid
        $this->messages .= $this->update_raid($raidId,$raid_array[$raidId]);
        $this->raidtracker_debug("Memory Usage: " . round((memory_get_usage()/1048576),2) . " MB",1);
        }
    unset($msg);
    unset($raid_array);
    unset($data_raid);
    }

    /**
     * Updates a raid in the database
     *
     * Step 1) Validate whether the raid already exists.
     *         1a) If it does, it is referenced for the building of all
     *             following queries.
     *         1b) otherwise we inject a new raid and associate all additional
     *           data with it.
     * Step 2) Call out to member functions to prepare the SQL injections
     *           and store them in the $sqls variable.
     *
     * Step 3) Inject the sql queries.
     *
     * Step 4) Return to the raid parsing function.
     *
     * For the future, this can be accomplished with transactional database
     * entries and the transaction_id to limit the possibility of single-point
     * failure or data corruption.
     *
     * @param array $raiddata
     */
    function update_raid(&$raidId,&$raiddata) {
        global $roster, $update;

	$newid = 0;    // Whether we received a new id
  	$newflag = 0;    // Whether we're storing a new raid

        // Debug
        // $this->raidtracker_debug(var_export($raiddata, true) .'<br>',2);

	// First let's split out the zone
        $raidzone = isset($raiddata['zone']) ? $raiddata['zone'] : 'Unknown';

	// Verify that we want to store this raid based on the Unknown/Old/BC/WLK
	// settings. This is split to function for readability.
   	 if(!$this->check_valid_raidzone($raidzone))
		 return $roster->locale->act['rt_ignored_byzone']."<br>";

    	// Now we've passed the zone check. Make sure and slash the zone name
    	$raidzone = addslashes($raidzone);

    	// Now split out the rest of the data
        $raidkey = date("Y-m-d H:i:s", strtotime($raiddata['key']));
        $raidday = date("Y-m-d", strtotime($raiddata['key']));
        // Can also be 'raidEnd'
        $raidend = isset($raiddata['End']) ? date("Y-m-d H:i:s", strtotime($raiddata['End'])) : (isset($raiddata['raidEnd']) ? date("Y-m-d H:i:s", strtotime($raiddata['raidEnd'])) : '0000-00-00 00:00:00');
        $raidnote = isset($raiddata['Notes']) ? $raiddata['Notes'] : '';
	$nextboss = isset($raiddata['NextBoss']) ? addslashes($raiddata['NextBoss']) : '';
	$wipes = isset($raiddata['wipes']) ? sizeof($raiddata['wipes']) : 0;
        $raidinstanceid = isset($raiddata['instanceid']) ? $raiddata['instanceid'] : 'None';
        // Legacy : Player info
        // Name => array( 'race' => race, 'guild' => guild, 'level' => XX )
        $raidplayerinfo = isset($raiddata['PlayerInfos']) ? $raiddata['PlayerInfos'] : array();
        // Raid leaves
        // array( ID => ( 'player' => name, 'time' => MM/DD/YY HH:MM:SS ))
        $raidleaves = isset($raiddata['Leave']) ? $raiddata['Leave'] : array();
        // Bosses
        // array ( ID => array( 'boss', 'time', 'attendees' => array()) )
        $raidbosses = isset($raiddata['BossKills']) ? $raiddata['BossKills'] : array();
        // Joins
        // array ( ID => array( 'player', 'time' ) )
        $raidjoins = isset($raiddata['Join']) ? $raiddata['Join'] : array();
        // Loot
        // array ( ID => array( 'boss', 'time', array('attendees'), 'player', 'item' = array( 'color', 'id', 'tooltip' = array(), 'count', 'subclass', 'class', 'icon', 'type, 'name'), zone )
        // if type is set, subclass and class are not
        $raidloot = isset($raiddata['Loot']) ? $raiddata['Loot'] : array();

        // Summary
        $this->raidtracker_debug('<ul>Raid Summary:
                    <ul>Key: '.$raidkey.'</ul>
                    <ul>Day: '.$raidday.'</ul>
                    <ul>End: '.$raidend.'</ul>
                    <ul>Zone: '.$raidzone.'</ul>
                    <ul>Notes: '.$raidnote.'</ul>
                    <ul>ID: '.$raidinstanceid.'</ul>
                    <ul># Players: '.sizeof($raidplayerinfo).'</ul>
                    <ul>Joins: '.sizeof($raidjoins).'</ul>
                    <ul>Leaves: '.sizeof($raidleaves).'</ul>
                    <ul># Bosses: '.sizeof($raidbosses).'</ul>
                    <ul># Loots: '.sizeof($raidloot).'</ul>
            <ul>Next Boss: '.$nextboss.'</ul>
            <ul>Wipes: '.$wipes.'</ul>
                    </ul>',0);

	$this->tbosses += sizeof($raidbosses);
	$this->tloots += sizeof($raidloot);
	$this->twipes += $wipes;
    
	// Verify whether this raid exists
	$exists = $this->raid_exists($raidinstanceid, $raidkey, $raidzone);
	if($exists == false) {
		// This is a new raid. Let's create our raid record, and
		// retrieve the record ID.
		$newflag = 1;
		$q = "INSERT INTO `".$roster->db->table('raids','raidtracker') . "` VALUES ".
		    "(NULL, '".$raidkey."','".$raidinstanceid."','".$raidend."','".$raidzone."','".$raidnote."',0,'".$this->server.
		    "','".$this->region."','".$this->guild."','".$this->faction."','".$wipes."');";
		$result = $roster->db->query($q);
		if(!$result) {
		    $this->messages .= "<strong>".$roster->locale->act['rt_fail_newraid']."</strong><br>";
		    $this->messages .= $roster->db->error()."<br>";
		    return false;
		} else {
		    $newid = $roster->db->insert_id();
		    $this->raidtracker_debug(str_replace("%n", $newid, $roster->locale->act['rt_success_newraid'])."<br>",1);
		    $roster->db->free_result($result);
		}
    	} else {
		// This is an existing raid
		$newflag = 0;
		$newid = $exists;
	}
    	$this->update_wipes($newflag,$newid,$wipes);
    	$this->process_loot($newflag,$newid,$raidloot);
        $this->process_joins($newflag,$newid,$raidjoins);
        $this->process_leaves($newflag,$newid,$raidleaves);
        $this->process_players($newflag,$newid,$raidplayerinfo);
        $this->process_bosses($newflag,$newid,$raidbosses);
        
        //$this->raidtracker_debug("<br>SQL strings:<br>".var_export($this->sqls,true)."<br>",2);
        
        // Unset all the variables
        unset($raidzone,$raidkey,$raidday,$raidend,$raidnote,$nextboss,$wipes,$raidinstanceid,$raidplayerinfo,$raidleaves,$raidbosses,$raidjoins,$raidloot,$exists,$q,$result,$newflag);
        // And return with message
        return str_replace("%r", $newid, $roster->locale->act['rt_raid_processed']).'<br>';
    }

    /**
    * function: update_wipes(newflag,newid,wipecount)
    * If this is not a new raid, we need to add the new wipes to it
    *
    * @param newflag bool
    * @param raidid int - raidid
    * @param wipecount int - number of wipes
    */
    function update_wipes(&$newflag,&$raidid,&$wipecount) {
        global $roster;
	
	if($newflag) return;
	
	return;
    }

    /**
    * function: check_valid_raidzone(zone)
    * Find out whether a raid zone is allowed to be stored
    *
    * @param zone string
    */
    function check_valid_raidzone(&$zone) {
        global $roster;
        
        // perform the unknown check
        if(!$this->data['config']['raidtracker_storeunknown'] && ($zone == "Unknown"))
            return false;
        
        // Now parse through the zones and perform checks
        $scan = $roster->locale->act['raidtracker_zones'];
        foreach(array_keys($scan) as $zonen) {
            // If the zone matches, perform checks
            if($zone == $zonen) {
                // Pull the xpac listing
                $xpac = $scan[$zonen]['pack'];
                // Check it against the config
                if(!$this->data['config']['raidtracker_storeoldworld'] && $xpac == "old") {
                    $this->raidtracker_debug("<ul>".$roster->locale->act['rt_ignored_old']."</ul>",1);
            unset($scan,$xpac);
                    return false;
                }
                if(!$this->data['config']['raidtracker_storebc'] && $xpac == "bc") {
                    $this->raidtracker_debug("<ul>".$roster->locale->act['rt_ignored_bc']."</ul>",1);
            unset($scan,$xpac);
                    return false;
                }
                if(!$this->data['config']['raidtracker_storewlk'] && $xpac == "wlk") {
                    $this->raidtracker_debug("<ul>".$roster->locale->act['rt_ignored_wlk']."</ul>",1);
            unset($scan,$xpac);
                    return false;
                }
                if(!$this->data['config']['raidtracker_storepvp'] && $xpac == "pvp") {
                    $this->raidtracker_debug("<ul>".$roster->locale->act['rt_ignored_pvp']."</ul>",1);
                    unset($scan,$xpac);
                    return false;
                }
            }
        }
	unset($scan,$xpac);
        return true;
    }

    /**
    * function: raid_exists(raidid, key, zone)
    * Find out whether a raid ID already exists
    * Note: This verifies against 2 data sets in this order:
    *        o InstanceID (if exists), guild and realm
    *        o Raid Key (date) within 4 hours and Zone, guild and realm
    *
    * @param raidid date
    * @param key date
    * @param zone string
    */
    function raid_exists(&$raidid, &$key, &$zone) {
        // Check to make sure we have raid merging enabled. If not, we
        // want to insert a new raid, so we return false
        if(!$this->data['config']['raidtracker_mergeraids']) return false;

        global $roster;

        // Next see if the raidid exists for the guild and server
        $this->raidtracker_debug("<ul>Seeking existing raid with raidid: ".$raidid.", Key: ".$key.", Zone: ".$zone.", Guild: ".$this->guild.", Server: ".$this->server."</ul>",1);

	//$q = "SELECT `raidID` FROM `". $roster->db->table('raids','raidtracker') . "` WHERE `instanceID` = '$raidid' AND `guild_name` = '".$this->guild."' AND `realm` = '".$this->server."' AND `raidtime` = '$key' AND `zone` = '$zone';";
	if($raidid != 0) $q = "SELECT `raidID` FROM `". $roster->db->table('raids','raidtracker') . "` WHERE `instanceID` = '$raidid' AND `guild_name` = '".$this->guild."' AND `realm` = '".$this->server."' AND `zone` = '$zone';";
        	else $q = "SELECT `raidID` FROM `". $roster->db->table('raids','raidtracker') . "` r WHERE `guild_name` = '".$this->guild.
			"' AND `realm` = '".$this->server.
			"' AND ( ABS(TIMEDIFF('".$key."', r.raidtime)) < '".$this->data['config']['raidtracker_timediff'].
			"' ) AND `zone` = '$zone';";

        //$this->messages .= "Raid exists query: ".$q."<br>";
        $result = $roster->db->query($q);

        if( !$result ) {
		$this->raidtracker_debug("<ul>".str_replace("%r",$raidid,$roster->locale->act['rt_fail_byraidid'])."<br>".
		    str_replace("%n", $this->data['config']['raidtracker_timediff'], $roster->locale->act['rt_seek_bytime'])."</ul>",1);
		// Now attempt to find by key and zone
		$q2 = "SELECT `raidID`, HOUR(TIMEDIFF(DATE_ADD(raidtime, INTERVAL ".$this->data['config']['raidtracker_mergelimit']." HOUR),'".$key."')) AS mergediff".
			" FROM `".$roster->db->table('raids','raidtracker') . "` r WHERE ".
		    "( ABS(TIMEDIFF('".$key."', r.raidtime)) < '".$this->data['config']['raidtracker_timediff']."' )".
		    " AND `zone` = '".$zone."' AND `guild_name` = '".$this->guild."' AND `realm` = '".$this->server."';";
		    //$this->messages .= "Raid exists query2: ".$q2."<br>";
	
		$res2 = $roster->db->query($q2);
		if( !$res2 ) {
		    $this->raidtracker_debug("<ul>".$roster->locale->act['rt_seek_fail']."</ul>",1);
		    unset($result,$q,$q2);
		    return false;
		} else {
		    // Now verify that we didn't get more than one hit
		    $num = $roster->db->num_rows($res2);
		    $msg = str_replace("%n",$num,$roster->locale->act['rt_seek_successbykey']);
		    $msg = str_replace("%r",$raidid,$msg);
		    $this->raidtracker_debug("<ul>".$msg."</ul>",1);
		    if($num > 1) {
			$this->raidtracker_debug("<ul>".$roster->locale->act['rt_seek_multi']."</ul>",1);
			$roster->db->free_result($res2);
			unset($q,$result,$q2,$res2,$msg,$num);
			return false;
		    } else {
			$myinfo = $roster->db->fetch($res2,SQL_ASSOC);
			$myid = $myinfo['raidID'];
			$roster->db->free_result($res2);
			unset($q,$result,$q2,$res2,$msg,$num,$myinfo);
			return $myid;
		    }
		}
        } else {
		$myinfo = $roster->db->fetch($result,SQL_ASSOC);
		$myid = $myinfo['raidID'];
		$roster->db->free_result($result);
		// Now verify that we're within raidtracker_mergelimit if it is
		// not set to 0. If we are, return false. (New id)
		if( ($this->data['config']['raidtracker_mergelimit'] != '0')
			&& ($myinfo['mergediff'] >= $this->data['config']['raidtracker_mergelimit'])) {
			$msg = str_replace("%r",$raidid,$roster->locale->act['rt_seek_idfail_mergelimit']);
			$this->raidtracker_debug("<ul>".$msg."</ul>",1);
			unset($q,$result,$q2,$res2,$msg,$num,$myinfo,$myid);
			return false;
		} else {
			$msg = str_replace("%r",$raidid,$roster->locale->act['rt_seek_successbyid']);
			$msg = str_replace("%n",$myid, $msg);
			$this->raidtracker_debug("<ul>".$msg."</ul>",1);
			unset($q,$result,$q2,$res2,$msg,$num,$myinfo);
			return $myid;
		}
	}
    }

    /**
    * Processes loot messages
    */
    // Loot
    // array ( ID => array( 'boss', 'time', array('attendees'), 'player',
    //                      'item' = array( 'color', 'id',
    //                                      'tooltip' = array(),
    //                        'count', 'subclass', 'class',
    //                        'icon', 'type, 'name'),
    //                 zone )
    // if type is set, subclass and class are not - type is the old style
    // Here's the breakout:
    // 15 => array (
    //            'player' => 'Gaxme', )
    //            'item' => array (
    //                'c' => 'ff1eff00',
    //                'id' => '19704:0:0:0',
    //                'tooltip' => array ( ),
    //                'count' => '1',
    //                'name' => 'Sandfury Coin',
    //                'subclass' => 'Quest',
    //                'icon' => 'INV_Misc_Coin_15',
    //                'class' => 'Quest',
    //        ),
    //            'zone' => 'Zul\'Gurub',
    //          'boss' => 'Trash mob',
    //            'time' => '09/13/06 20:05:39',
    //            'attendees' => array ( ),
    function process_loot(&$new,&$raidid,&$loot) {
        // Validate that we're configured to store items
        if(!$this->data['config']['raidtracker_storeitems']) {
            $this->messages .= "<ul>".$roster->locale->act['rt_nostore_items']."</ul>";
            return;
        }

        global $update, $roster;
		$deepscan = false;
		$lootid = false;
		$inscount = 0;

        // Verify the array actually has size, and store that size for item checking
        $totalloot = sizeof($loot);
		if($totalloot > 0) {
			if($new) $this->raidtracker_debug("<ul>".str_replace("%r",$raidid,$roster->locale->act['rt_items_process_new'])."</ul>",0);
				else $this->raidtracker_debug("<ul>".str_replace("%r",$raidid,$roster->locale->act['rt_items_process_old'])."</ul>",0);
			// Next we need to make reasonably sure that, if this is not a
			// new raid, we're not duplicating loot. Start this process by
			// line-item count.
			if(!$new) {
				$q = "SELECT COUNT(lootID) as storedLoot FROM `".
				 $roster->db->table('items',$this->data['basename']) .
				 "` WHERE `raidFK` = '".$raidid."';";
				$res = $roster->db->query_first($q);
				if(isset($res) && !empty($res['storedLoot'])) {
				if($res['storedLoot'] == $totalloot) {
					$this->raidtracker_debug("<ul>".$roster->locale->act['rt_items_skipped_count']."</ul>",0);
	//                return;
				} else {
					// Set the deepscan flag to compare items.
					$deepscan = 1;
				}
			}
		}
        // Now loop through the loot array
        foreach( array_keys($loot) as $lootid ) {
            // Pull out the item array
            $iteminfo = $loot[$lootid]['item'];
            // Pull out the color for rarity check
            $color = $iteminfo['c'];
            $htmlc = "#".substr($color,2);
    
            // Validate that we will store this item quality
            if($this->check_rarity($color)) {
            // Now break out the rest of the item info
            $name = $loot[$lootid]['player'];
            $zone = isset($loot[$lootid]['zone']) ? addslashes($loot[$lootid]['zone']) : "Unknown";
            $loottime = date("Y-m-d H:i:s", strtotime($loot[$lootid]['time']));
            $boss = isset($loot[$lootid]['boss']) ? addslashes($loot[$lootid]['boss']) : "None";
            $itemid = $iteminfo['id'];
            // $tooltip = addslashes($iteminfo['tooltip']);
            $numitem = $iteminfo['count'];
            $itemname = isset($iteminfo['name']) ? addslashes($iteminfo['name']) : "";
            $itemicon = isset($iteminfo['icon']) ? strtolower($iteminfo['icon']) : "inv_misc_questionmark";
            $itemtype = isset($iteminfo['type']) ? $iteminfo['type'] : $iteminfo['class'];
            $note = isset($loot[$lootid]['note']) ? addslashes($loot[$lootid]['note']) : "";
            $players = !empty($loot[$lootid]['attendees']) ? $loot[$lootid]['attendees'] : array();
            // This allows internal checking against loot
            $ctrtid = isset($loot[$lootid]['thisitemid']) ? $loot[$lootid]['thisitemid'] : 0;
            
            $msg = str_replace("%p", $htmlc, $roster->locale->act['rt_item_rarity_pass']);
            $msg = str_replace("%i", $itemname, $msg);
            $this->raidtracker_debug("<ul>".$msg."</ul>",1);
            // If we're set to deepscan, then compare the items
            if(isset($deepscan)) {
                $q = "SELECT `lootID` FROM `" . $roster->db->table('items',$this->data['basename']) .
                     "` WHERE `raidFK` = '".$raidid."' AND `itemref` = '".$itemid."' AND `zone` = '".
                     $zone."' AND `boss` = '".$boss."' AND `number` = '".$numitem."' AND `color` = '".
                     $color."' AND `member` = '".$name."' AND `loottime` = '".
                     $loottime."' AND `loadnum` = '".$ctrtid."';";
                $result = $roster->db->query_first($q);
                if($result) {
                    $msg = str_replace("%p", $htmlc, $roster->locale->act['rt_item_exists']);
                    $msg = str_replace("%i", $itemname, $msg);
                    $this->raidtracker_debug('<ul>'.$msg.'</ul>',1);
                    //$roster->db->free_result($result);
                    if(sizeof($players) && $this->data['config']['raidtracker_storeplayers'])
                        $this->process_loot_attendees($new,$result,$players,$raidid);
                    continue 1;
                }
            }
            // Otherwise:
            // We've passed the checks, let's insert the data
            // and retrieve and ID
            $update->reset_values();
            $update->add_value('raidFK', $raidid);
            $update->add_value('itemname', $itemname);
            $update->add_value('zone', $zone);
            $update->add_value('boss', $boss);
            $update->add_value('number', $numitem);
            $update->add_value('color', $color);
            $update->add_value('loottime', $loottime);
            $update->add_value('member', $name);
            $update->add_value('note', $note);
            $update->add_value('itemref', $itemid);
            $update->add_value('icon', $itemicon);
            $update->add_value('loadnum', $ctrtid);
    
            $querystr = "INSERT INTO `" . $roster->db->table('items',$this->data['basename']) . "` SET " . $update->assignstr . ';';
            $result = $roster->db->query($querystr);
            if( !$result ) {
                $this->messages .= "<ul>".$roster->locale->act['rt_item_insert_fail']."</ul>";
                $this->messages .= $roster->db->error();
                continue;
            } else {
                $lootid = $roster->db->insert_id();
                $msg = $roster->locale->act['rt_item_rarity_pass'];
                $msg = str_replace("%p", $htmlc, $msg);
                $msg = str_replace("%i", $itemname, $msg);
                $this->raidtracker_debug("<ul>".$msg."</ul>",1);
                $inscount = $inscount + 1;
                $roster->db->free_result($result);
                if(sizeof($players) && $this->data['config']['raidtracker_storeplayers'])
                    $this->process_loot_attendees($new,$lootid,$players,$raidid);
            }
            } else { // End passed rarity check
            $msg = $roster->locale->act['rt_item_rarity_fail'];
            $msg = str_replace("%p", $htmlc, $msg);
            $msg = str_replace("%i", $iteminfo['name'], $msg);
            $this->raidtracker_debug("<ul>".$msg."</ul>",1);
            continue;
            } // End rarity check
        } // End loot loop
        } else { // End existing loot segment
        $this->raidtracker_debug('<ul>'.str_replace("%r",$raidid,$roster->locale->act['rt_no_items']).'</ul>',0);
        unset($deepscan,$lootid, $inscount,$totalloot,$q,$res,$iteminfo,$color,$htmlc,$name,$zone,$loottime,$boss,$itemid,$numitem,$itemname,$itemicon,$itemtype,$note,$players,$ctrtid,$msg,$result,$querystr);
        return;
    } // End loot segment
    // Insert data into the database if it exists
    $this->raidtracker_debug("<ul>".str_replace("%n", $inscount, $roster->locale->act['rt_insert_item_count'])."</ul>",0);
    unset($deepscan,$lootid, $inscount,$totalloot,$q,$res,$iteminfo,$color,$htmlc,$name,$zone,$loottime,$boss,$itemid,$numitem,$itemname,$itemicon,$itemtype,$note,$players,$ctrtid,$msg,$result,$querystr);
    $this->inject_database();
    return;
    }

    /**
    * Function to process loot attendees
    * @param new
    * @param lootid
    * @param players array
    * @param raidid
    */
    function process_loot_attendees(&$new,&$lootid,&$players,&$raidid) {
    global $roster, $update;

    // If the tracker is configured to store players present at time of loot, record it
    if($this->data['config']['raidtracker_storeplayers']) {
        $msg = $roster->locale->act['rt_item_attend_attempt'];
        $msg = str_replace("%n", $lootid, $msg);
        $msg = str_replace("%r", $raidid, $msg);
        $this->raidtracker_debug("<ul>".$msg,1);

        // Per-item attendance checking is costly. If we reach this
        // point we've got attendance data. Until someone complains,
        // we're just going to wipe out existing data and put in new.
        // If the raid is not new, pull the old data
        if(!$new) {
            $q = "DELETE FROM `".$roster->db->table('itemattend',$this->data['basename'])."` WHERE `raidFK` = '".$raidid."' AND `lootFK` = '".$lootid."';";
            $res = $roster->db->query($q);
            if(!$res) {
                $this->messages .= "<ul>".$roster->locale->act['rt_db_fail_itemattend']."</ul>";
                $this->messages .= $roster->db->error();
                //return;
            }
            $roster->db->free_result($res);
        }
        // Loop through the players and add the data.
        foreach( array_keys($players) as $aid ) {
            $aname = $players[$aid];
            //$this->messages .= "Itemattend: Player ".$aname." does not exist in the database.<br>";
            $update->reset_values();
            $update->add_value('raidFK',$raidid);
            $update->add_value('lootFK', $lootid);
            $update->add_value('member',addslashes($aname));
            $this->sqls[] = "INSERT INTO `" . $roster->db->table('itemattend',$this->data['basename']) . "` SET " . $update->assignstr . ";";
        }
        $msg = $roster->locale->act['rt_item_attend_store'];
        $msg = str_replace("%n", sizeof($players), $msg);
        $msg = str_replace("%i", $lootid, $msg);
        $msg = str_replace("%r", $raidid, $msg);
        $this->raidtracker_debug("<ul>".$msg."</ul></ul>",1);
        $this->inject_database();
    } else {
        $this->raidtracker_debug("<ul>".$roster->locale->act['rt_storeplayers_disabled']."</ul>",0);
    }
    unset($msg,$q,$res,$aname);
    return true;
    }
    
    /**
    * Internal function to check whether we store an item by rarity
    */
    function check_rarity($color) {
        global $roster;
        $types = $roster->locale->act['lootTypes'];
        $minrarity = $this->data['config']['raidtracker_minitemlvl'];
        foreach( array_keys($types) as $ckey ) {
			// Extract the color we're currently working with
			$workc = $types[$ckey]['color'];
			$this->raidtracker_debug("<ul>Working color:".$workc."</ul><br>",2);
			if($workc == $color) {
				if($ckey < $minrarity) {
					unset($types,$minrarity,$workc);
					return false;
				}
				else {
					unset($types,$minrarity,$workc);
					return true;
				}
			}
        }
        // If we reached this we didn't get a proper color
		$msg = $roster->locale->act['rt_rarity_error'];
		$msg = str_replace("%i", "#".substr($color,2), $msg);
		$msg = str_replace("%p", $color, $msg);
		$this->raidtracker_debug("<ul>".$msg."</ul>",0);
		unset($msg,$types,$minrarity,$workc);
        return false;
    }

    /**
    * Processes join messages
    */
    function process_joins(&$new, &$raidid, &$join) {
        // Validate that attendance tracking is enabled
        if(!$this->data['config']['raidtracker_storeattend']) {
        $this->messages .= "<ul>".$roster->locale->act['rt_attend_disabled']."</ul>";
        return;
        }
        // Verify we have data size
        if(!sizeof($join)) return;

            global $roster, $update;

        $ojoin = array();
        // Pull the existing leave data for the raidid if it is NOT new
        if(!$new) {
        $q = "SELECT * FROM `".$roster->db->table('raidjoins',$this->data['basename'])."` WHERE `raidFK` = '".$raidid."';";
        $res = $roster->db->query($q);
        if(!$res) {
            $this->messages .= "<ul>".$roster->locale->act['rt_db_fail_joins']."</ul>";
            $this->messages .= $roster->db->error();
        }
        $ojoin = $roster->db->fetch_all($res);
        $roster->db->free_result($res);
        }
        // This is uncomplicated, it's a table entry per line with a
        // comparison to the existing data
        foreach( array_keys($join) as $jid ) {
        $time = date("Y-m-d H:i:s", strtotime($join[$jid]['time']));
        $player = $join[$jid]['player'];

        if(!$new) {
            foreach( array_keys($ojoin) as $oid ) {
                if(stripslashes($ojoin[$oid]['member']) == $player) {
//                    $this->messages .= "<ul>Joins: Comparing stored ".$ojoin[$oid]['member']." to ".$player." and ".
//                        $ojoin[$oid]['datejoin']." to ".$time."</ul>";
                    if($ojoin[$oid]['datejoin'] == $time)
                        continue 2;
                }
            }
        }
        $update->reset_values();
        $update->add_value('raidFK',$raidid);
        $update->add_value('datejoin', $time);
        $update->add_value('member', addslashes($player));

        $this->sqls[] = "INSERT INTO `" . $roster->db->table('raidjoins',$this->data['basename']) . "` SET " . $update->assignstr . ";";
        
        }
        $this->messages .= "<ul>".str_replace("%n", sizeof($this->sqls), $roster->locale->act['rt_joins_insert']."</ul>");
        unset($ojoin,$q,$res,$time,$player);
        $this->inject_database();
        return;
    }

    /**
    * Processes leave messages
    */
    // Data structure:
    //   ['Leave'] = array( "time" => "MM/DD/YY HH:MM:SS",
    //                "player" => "Name",
    //                )
    function process_leaves(&$new, &$raidid, &$leave) {        
        // Validate that attendance tracking is enabled
        if(!$this->data['config']['raidtracker_storeattend']) {
        $this->messages .= "<ul>".$roster->locale->act['rt_attend_disabled']."</ul>";
        return;
        }
        if(!$this->data['config']['raidtracker_playerleave']) {
        $this->messages .= "<ul>".$roster->locale->act['rt_leave_disabled']."</ul>";
        return;
        }
        // Verify we have data size
        if(!sizeof($leave)) return;

        global $roster, $update;

        $oleaves = array();
        // Pull the existing leave data for the raidid if it is NOT new
        if(!$new) {
        $q = "SELECT * FROM `".$roster->db->table('raidleaves',$this->data['basename'])."` WHERE `raidFK` = '".$raidid."';";
        $res = $roster->db->query($q);
        if(!$res) {
            $this->messages .= "<ul>".$roster->locale->act['rt_db_fail_leaves']."</ul>";
            $this->messages .= $roster->db->error();
        }
        $oleaves = $roster->db->fetch_all($res);
        $roster->db->free_result($res);
        }
        // This is uncomplicated, it's a table entry per line with a
        // comparison to the existing data
        foreach( array_keys($leave) as $lid ) {
        $time = date("Y-m-d H:i:s", strtotime($leave[$lid]['time']));
        $player = $leave[$lid]['player'];

        if(!$new) {
            foreach( array_keys($oleaves) as $oid ) {
                if($oleaves[$oid]['member'] == $player) {
//                    $this->messages .= "<ul>Leaves: Comparing stored ".$oleaves[$oid]['member']." to ".$player." and ".
//                                $oleaves[$oid]['dateleft']." to ".$time."</ul>";
                    if($oleaves[$oid]['dateleft'] == $time)
                        continue 2;
                }
            }
        }
        $update->reset_values();
        $update->add_value('raidFK',$raidid);
        $update->add_value('dateleft', $time);
        $update->add_value('member', addslashes($player));

        $this->sqls[] = "INSERT INTO `" . $roster->db->table('raidleaves',$this->data['basename']) . "` SET " . $update->assignstr . ";";
        
        }
        $this->messages .= "<ul>".str_replace("%n", sizeof($this->sqls), $roster->locale->act['rt_leaves_insert']."</ul>");
        unset($oleaves,$q,$res,$time,$player);
        $this->inject_database();
        return;
    }

    /**
    * Processes player infos. Update the information if it exists. The
    *    player's name is the primary key
    */
    // Data structure:
    //   PlayerInfos = array( "CharName" => array(
    //              "race" => "RaceName",
    //              "guild" => "GuildName",
    //              "sex" => "sexID" (2 = male, 3 = female)
    //              "class" => "CLASS",
    //              "level" => num, )
    //                )
    function process_players(&$new, &$raidid, &$players) {
        global $roster, $update;

    // First let's do one query to pull all the player infos that exist
    // to compare against. This is more streamlined than pulling every time
    // for comparison.
    $q = "SELECT * FROM `".$roster->db->table('raidmembers',$this->data['basename'])."` ORDER BY name ASC;";
    $res = $roster->db->query($q);
    if(!$res) {
        $this->messages .= "<ul>Failed to retrieve existing player data!</ul>";
        $this->messages .= $roster->db->error();
    }
    $fetch = $roster->db->fetch_all($res,SQL_BOTH);
    $roster->db->free_result($res);
    // Split the old data into the same array type
    $odata = array();
    $mcount = 0;
    foreach( array_keys($fetch) as $oid ) {
        $odata[$fetch[$oid]["name"]] = array(
            "race" => $fetch[$oid]["race"],
            "guild" => $fetch[$oid]["guild_name"],
            "class" => $fetch[$oid]["class"],
            "level" => $fetch[$oid]["level"],
            "realm" => $fetch[$oid]["realm"],
            "region" => $fetch[$oid]["region"],
            "faction" => $fetch[$oid]["faction"],
            );
    }
    foreach( array_keys($players) as $pname ) {
        // Does the player already exist. If so, compare and if diff
        // prepare an update. Otherwise, less SQL calls faster!
        $urace = $players[$pname]["race"];
        $uguild = isset($players[$pname]["guild"]) ? $players[$pname]["guild"] : $this->guild;
        $uclass = ucwords($players[$pname]["class"]);
        $ulevel = isset($players[$pname]["level"]) ? $players[$pname]["level"] : 0;
        $urealm = isset($players[$pname]["realm"]) ? $players[$pname]["realm"] : $this->server;
        $uregion = isset($players[$pname]["region"]) ? $players[$pname]["region"] : $this->region;
        $ufaction = isset($players[$pname]["faction"]) ? $players[$pname]["faction"] : $this->faction;
        // If the player name exists, then we want to update their
        // level and guild
        if(!empty($odata[$pname])) {
            $orace = $odata[$pname]["race"];
            $oguild = $odata[$pname]["guild"];
            $oclass = ucwords($odata[$pname]["class"]);
            $olevel = $odata[$pname]["level"];
            $orealm = $odata[$pname]["realm"];
            $oregion = $odata[$pname]["region"];
            $ofaction = $odata[$pname]["faction"];
            // Now do the comparisons. If race, class, realm,
            // region and faction match, then we should update IF
            // the level is larger or the guild has changed.
            // We have to be more careful of this now, since
            // roster can handle multiple guilds on a realm
            if( $urace == $orace && $uclass == $oclass &&
                $urealm == $orealm && $uregion != $oregion &&
                $ufaction == $ofaction &&
                ($ulevel > $olevel || $uguild != $oguild ) ) {

                $update->reset_values();
                $update->add_value('level', $ulevel);
                $update->add_value('guild_name', $uguild);
                $this->sqls[] = "INSERT INTO `" . $roster->db->table('raidmembers',$this->data['basename']) . "` SET " . $update->assignstr . " WHERE `name` = '".$pname."';";
                $mcount += 1;
            }
        } else {
            // We enter a new player
            $update->reset_values();
            $update->add_value('name', ucwords($pname));
            $update->add_value('race', $urace);
            $update->add_value('class', $uclass);
            $update->add_value('level', $ulevel);
            $update->add_value('guild_name', $uguild);
            $update->add_value('realm', $urealm);
            $update->add_value('region', $uregion);
            $update->add_value('faction', $ufaction);
            $this->sqls[] = "INSERT INTO `" . $roster->db->table('raidmembers',$this->data['basename']) . "` SET " . $update->assignstr . ';';
            $mcount += 1;
        }
    }
    // Now inject the sql calls
    $this->raidtracker_debug("<ul>".str_replace("%n", $mcount, $roster->locale->act['rt_insert_players_count'])."</ul>",0);
    unset($q,$res,$fetch,$odata,$urace,$uguild,$uclass,$ulevel,$urealm,$uregion,$ufaction,$orace,$oguild,$oclass,$olevel,$orealm,$oregion,$ofaction,$mcount);
    $this->inject_database();
        return;
    }
    
    /**
    * Processes bosses
    */
    // Data structure:
    //   BossKills = array( "boss" => "Name",
    //                      "time" => "MM/DD/YY HH:MM:SS",
    //                "attendees" => array(
    //                "Name1", "Name2",
    //                ),
    //                )
    function process_bosses(&$new, &$raidid, &$bosses) {
	    // Validate that boss tracking is enabled
	    if(!$this->data['config']['raidtracker_storebosses']) {
		$this->messages .= "<ul>".$roster->locale->act['rt_bosses_disabled']."</ul>";
		return;
	    }
	    // Verify we have data size
	    if(!sizeof($bosses)) {
		$this->messages .= "<ul>Bosses: No data!</ul>";
		return;
	    }
	    global $roster, $update;
	
	    // Bosses entered counter
	    $bcount = 0;
	    $bossnames = array();
	
	    // If not new, pull the old boss data
	    if(!$new) {
		$q = "SELECT * FROM `".$roster->db->table('bosskills',$this->data['basename'])."` WHERE `raidFK` = '".$raidid."';";
		$res = $roster->db->query($q);
		if(!$res) {
		    $this->messages .= "<ul>".$roster->locale->act['rt_db_fail_bosses']."</ul>";
		    $this->messages .= $roster->db->error();
		}
		$obdata = $roster->db->fetch_all($res);
		$roster->db->free_result($res);
		foreach( array_keys($obdata) as $oid ) {
		    $oname = $obdata[$oid]['boss'];
		    $otime = $obdata[$oid]['time'];
		    
		    $bossnames[$oname] = array( "time" => $otime, );
		}
	    }
	    // Loop the bosses and if not a new raid, verify that they are in
	    // the DB. If they are, we ignore them. If not, we add them.
	    foreach( array_keys($bosses) as $bid ) {
		// Extract the appropriate boss name, time, and attendees
		$time = date("Y-m-d H:i:s", strtotime($bosses[$bid]['time']));
		$boss = stripslashes($bosses[$bid]['boss']);
		$attend = $bosses[$bid]['attendees'];
	
		if(!$new) {
		    if( !in_array($boss, array_keys($bossnames)) ) {
			// Enter the data
			$bcount += 1;
			$this->raidtracker_debug("<ul>Bosses: Adding left-out ".$boss." to the database.</ul>",2);;
			$update->reset_values();
			$update->add_value('raidFK',$raidid);
			$update->add_value('time', $time);
			$update->add_value('boss', addslashes($boss));
			$this->sqls[] = "INSERT INTO `" . $roster->db->table('bosskills',$this->data['basename']) . "` SET " . $update->assignstr . ";";
		    } else {
			$this->raidtracker_debug("<ul>Bosses: ".$boss." already in the database for this raid.</ul>",2);
		    }
		} else {
		    $bcount += 1;
		    $this->raidtracker_debug("<ul>Bosses: Adding ".$boss." to the database.</ul>",2);
		    $update->reset_values();
		    $update->add_value('raidFK',$raidid);
		    $update->add_value('time', $time);
		    $update->add_value('boss', addslashes($boss));
		    $this->sqls[] = "INSERT INTO `" . $roster->db->table('bosskills',$this->data['basename']) . "` SET " . $update->assignstr . ";";
		}
		// Now see if there are attendees stored, and if we want them
		// stored in the database. If we do, wipe out the table and
		// insert them over again.
		if(sizeof($attend)) {
		    if(!$this->data['config']['raidtracker_storeattend']) {
			$this->raidtracker_debug("<ul>".$roster->locale->act['rt_bossattend_off']."</ul>",0);
			continue;
		    }
		    $acount = 0;
		    // Pull the old data and set it up first if this isn't
		    // a new raid.
		    if(!$new) {
			$this->raidtracker_debug("<ul>Bossattend: Not a new raid. Checking existing attendees.</ul>",1);
			$q = "DELETE FROM `".$roster->db->table('bossattend',$this->data['basename'])."` WHERE `raidFK` = '".$raidid."' AND `boss` = '".addslashes($boss)."';";
			$res = $roster->db->query($q);
			if(!$res) {
			    $this->messages .= "<ul>".$roster->locale->act['rt_db_fail_bossattend']."</ul>";
			    $this->messages .= $roster->db->error();
			}
			$roster->db->free_result($res);
		    }
		    foreach( array_values($attend) as $name) {
			$acount += 1;
			$this->raidtracker_debug("<ul>Bossattend: Player ".$name." added to boss attendance.</ul>",1);
			$update->reset_values();
			$update->add_value('raidFK',$raidid);
			$update->add_value('boss', addslashes($boss));
			$update->add_value('member',addslashes($name));
			$this->sqls[] = "INSERT INTO `" . $roster->db->table('bossattend',$this->data['basename']) . "` SET " . $update->assignstr . ";";
		    }
		    $msg = str_replace("%n", $acount, $roster->locale->act['rt_bossattend_insert']);
		    $msg = str_replace("%p", $boss, $msg);
		    $this->raidtracker_debug("<ul>".$msg."</ul>",0);
		}
	    }
	    $msg = str_replace("%n", $bcount, $roster->locale->act['rt_bosses_insert']);
	    $msg = str_replace("%r", $raidid, $msg);
	    $this->messages .= "<ul>".$msg."</ul>";
	    unset($bcount,$bossnames,$q,$res,$obdata,$time,$boss,$attend,$acount,$msg);
	    $this->inject_database();
	    return;
    }

    /**
     * Resets addon messages
     */
     // If the messages are reset, prepend the name of the addon
     function reset_messages()
    {
        $this->messages = '<strong>RaidTracker:</strong><br>';
    }

    /**
     * Displays debug messages if debug is enabled
     */
     function raidtracker_debug($msg,$level=2) {
         if($this->data['config']['raidtracker_debug'] >= $level)
             $this->messages .= $msg."\n";
         return;
     }
    
     /**
     * function inject_database()
     *
     * Takes the current $sqls array and executes the commands
     */
     function inject_database() {
         if(empty($this->sqls)) return;
         global $roster;
         foreach(array_keys($this->sqls) as $injectid) {
        // Future - enable transactions
        // $roster->db->begin_transaction;
        // These are very long processes, we're going to play with the execution time
        set_time_limit(30);
        $result = $roster->db->query($this->sqls[$injectid]);
        if(!$result) {
            $this->messages .= "<strong>Critical Error:</strong> SQL Injection failed!<br>";
            $this->messages .= $roster->db->error()."<br>";
            unset($result);
            return false;
        } else {
            //$this->raidtracker_debug("Successful SQL injection: ".$this->sqls[$injectid]."<br>",2);
        }
         }
         // Clear the current sqls queue
         unset($this->sqls,$result);
         $this->sqls = array();
         return true;
     }
}
?>
